import os
import tomllib
from pathlib import Path
from typing import Optional

from pydantic_settings import BaseSettings
from rich.console import Console
from rich.panel import Panel
from rich.text import Text

from maws.clients.ecs_service_deployment_client import AuthenticatedClient

from . import __app_name__, __version__

CONFIG_FILE_PATH = Path.home() / ".skaylink" / "profile.toml"


def show_config_help():
    console = Console()

    help_text = Text()
    help_text.append(f"Welcome to {__app_name__}! 🦆\n\n", style="bold cyan")
    help_text.append("To get started, create a configuration file at: ", style="white")
    help_text.append(str(CONFIG_FILE_PATH) + "\n\n", style="bold yellow")
    help_text.append("Example configuration:\n\n", style="white")
    help_text.append(
        """[profiles.dev]
API_BASE_URL = "https://dev-api.example.com"
API_ACCESS_TOKEN = "your-dev-token"

[profiles.prod]
API_BASE_URL = "https://prod-api.example.com"
API_ACCESS_TOKEN = "your-prod-token"
""",
        style="green",
    )
    help_text.append("\nThen use: ", style="white")
    help_text.append(f"{__app_name__} ecs deploy service image --profile dev", style="bold blue")

    console.print(Panel(help_text, title="Configuration Required", border_style="cyan"))


def load_profile(name: str = None) -> dict:
    if not name:
        return {}

    if not CONFIG_FILE_PATH.exists():
        show_config_help()
        raise SystemExit(1)

    with open(CONFIG_FILE_PATH, "rb") as f:
        data = tomllib.load(f)

    profiles = data.get("profiles", {})
    if name not in profiles:
        console = Console()
        console.print(f"Profile '{name}' not found in {CONFIG_FILE_PATH}", style="red")
        console.print(f"Available profiles: {', '.join(profiles.keys())}", style="yellow")
        raise SystemExit(1)

    return profiles.get(name)


class DotEnvSettings(BaseSettings):
    api_version: str = os.getenv("API_VERSION", "v1")
    api_base_url: Optional[str] = os.getenv("API_BASE_URL")
    api_access_token: Optional[str] = os.getenv("API_ACCESS_TOKEN")


class Settings(DotEnvSettings):
    meta: dict = {
        "name": __app_name__,
        "version": __version__,
    }

    def __init__(cls, profile: str = None):
        """
        Args:
            profile (str, optional): Defaults to None.
        """
        super().__init__()
        if profile:
            data = load_profile(name=profile)
            cls.api_base_url = data.get("API_BASE_URL")
            cls.api_access_token = data.get("API_ACCESS_TOKEN")
            if data.get("API_VERSION"):
                cls.api_version = data.get("API_VERSION")

    @property
    def api_client(cls) -> AuthenticatedClient:
        """
        Get API client

        Returns:
            AuthenticatedClient
        """
        return AuthenticatedClient(
            base_url=f"{cls.api_base_url}/{cls.api_version}",
            token=cls.api_access_token,
            auth_header_name="x-api-token",
            prefix="",
        )


def get_settings(profile: str = None):
    return Settings(profile=profile)


env = get_settings()



"""Contains all the data models used in inputs/outputs"""

from .error_response import ErrorResponse
from .service_deployment_request import ServiceDeploymentRequest
from .task_deployment_request import TaskDeploymentRequest

__all__ = (
    "ErrorResponse",
    "ServiceDeploymentRequest",
    "TaskDeploymentRequest",
)



import json
import time
from http import HTTPStatus

import typer
from rich.console import Console

from maws.clients.ecs_service_deployment_client.api.services import (
    get_service,
    patch_service,
)
from maws.clients.ecs_service_deployment_client.models import (
    ServiceDeploymentRequest,
)
from maws.config import CONFIG_FILE_PATH, get_settings

app = typer.Typer(no_args_is_help=True)
console = Console()


@app.command()
def deploy(
    service_name: str = typer.Argument(help="The name of the service to be updated"),
    image: str = typer.Argument(help="The container image to use for the service"),
    force: bool = typer.Option(
        False,
        help="Force new deployment, event if images has not changed",
    ),
    secret_arns: list[str] = typer.Option(
        [],
        help="List of secret ARNs to attach to the service",
    ),
    profile: str = typer.Option(None, help=f"Profile name from {str(CONFIG_FILE_PATH)}"),
) -> None:
    """
    ECS Service Deployment Request

    Args:
        service_name (str)
        image (str)
        force (str, Optional):  Defaults to False.
        secret_arns (list[str], Optional)
        profile (str, Optional): Profile name

    Raises:
        Exception
    """
    env = get_settings(profile)
    try:
        response = patch_service.sync_detailed(
            service=service_name,
            client=env.api_client,
            body=ServiceDeploymentRequest(image=image, force=force, secret_arns=secret_arns),
        )
        if response.status_code == HTTPStatus.CREATED:
            console.print(
                f"Deployment successfully started for service [italic]{service_name}[/italic]",
                style="green",
            )
            return status(service_name=service_name, delay=5, profile=profile)
        else:
            content = json.loads(response.content)
            console.print(f"[ERROR] {content.get("error")}", style="red", new_line_start=True)
            raise Exception(f"Deployment failed with status {response.status_code}")
    except Exception as e:
        console.print(e, overflow="fold", style="red")
        return typer.Abort()


@app.command()
def status(
    service_name: str = typer.Argument(help="Name of the ECS service to check"),
    delay: int = typer.Option(5, help="The delay to check if the ECS service status"),
    profile: str = typer.Option(help=f"Profile name from {str(CONFIG_FILE_PATH)}"),
) -> None:
    """
    Get the status of an ECS service deployment

    Args:
        service_name (str)
        delay (int, optional):  Defaults to 5.
        profile (str, Optional): Profile name

    Raises:
        Exception
    """
    env = get_settings(profile)
    console.print(
        f"Checking deployment status for service [italic]{service_name}[/italic]",
        end="",
        style="cyan",
    )
    try:
        while True:
            response = get_service.sync_detailed(service=service_name, client=env.api_client)
            match response.status_code:
                case HTTPStatus.ACCEPTED:
                    console.print(".", end="", style="cyan")
                case HTTPStatus.EXPECTATION_FAILED:
                    console.print(
                        f"\nDeployment failed with status {response.status_code}.",
                        style="red",
                    )
                    break
                case HTTPStatus.OK:
                    console.print(
                        f"\nDeployment succeeded with status {response.status_code}.",
                        style="green",
                    )
                    break
                case _:
                    raise Exception(f"\nDeployment failed with status {response.status_code}.")
            time.sleep(delay)
    except Exception as e:
        console.print(e, overflow="fold", style="red")
        return typer.Abort()



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceDeploymentRequest")


@_attrs_define
class ServiceDeploymentRequest:
    """ECS Service Deployment Request.

    Attributes:
        image (str): The container image to use for the service
        force (Union[None, Unset, bool]): Force deployment of same image Default: False.
        secret_arns (Union[None, Unset, list[str]]): List of secret ARNs to attach to the service
    """

    image: str
    force: Union[None, Unset, bool] = False
    secret_arns: Union[None, Unset, list[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        image = self.image

        force: Union[None, Unset, bool]
        if isinstance(self.force, Unset):
            force = UNSET
        else:
            force = self.force

        secret_arns: Union[None, Unset, list[str]]
        if isinstance(self.secret_arns, Unset):
            secret_arns = UNSET
        elif isinstance(self.secret_arns, list):
            secret_arns = self.secret_arns

        else:
            secret_arns = self.secret_arns

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "image": image,
            }
        )
        if force is not UNSET:
            field_dict["force"] = force
        if secret_arns is not UNSET:
            field_dict["secret_arns"] = secret_arns

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        image = d.pop("image")

        def _parse_force(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        force = _parse_force(d.pop("force", UNSET))

        def _parse_secret_arns(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                secret_arns_type_0 = cast(list[str], data)

                return secret_arns_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        secret_arns = _parse_secret_arns(d.pop("secret_arns", UNSET))

        service_deployment_request = cls(
            image=image,
            force=force,
            secret_arns=secret_arns,
        )

        service_deployment_request.additional_properties = d
        return service_deployment_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import typer

from maws.commands import ecs

# commands
ecs_commands = typer.Typer(no_args_is_help=True)
ecs_commands.add_typer(ecs.app)

app = typer.Typer(help="Welcome to Skaylink client.", no_args_is_help=True)
app.add_typer(ecs_commands, name="ecs", help="ECS management commands")


def main():
    app()  # pragma: no cover


if __name__ == "__main__":
    main()  # pragma: no cover
